/// <reference types="node" />
/**
 *
 * StateSwitch for easy managing the states with async programming
 *
 * Class StateSwitch
 * Licenst: Apache-2.0
 * https://github.com/huan/state-switch
 *
 * Helper Class for Manage State Change
 */
import { EventEmitter } from 'events';
import type { Loggable } from 'brolog';
import type { StateSwitchInterface, StateSwitchOptions } from './interfaces.js';
/**
 * Using Three Valued Logic for ON/OFF State
 *  https://github.com/huan/state-switch/issues/1
 *
 * Three-valued Logic (3VL): `true`, `false`, and
 *  'pending': it's in process, not stable.
 */
declare type Pending = 'pending';
declare type StateType = 'active' | 'inactive' | 'on' | 'off';
export declare class StateSwitch extends EventEmitter implements StateSwitchInterface {
    protected readonly _name: string;
    protected readonly _options: StateSwitchOptions;
    protected _log: Loggable;
    protected _activePromise: Promise<void>;
    protected _inactivePromise: Promise<void>;
    protected _activeResolver: Function;
    protected _inactiveResolver: Function;
    protected _isActive: boolean;
    protected _isPending: boolean;
    constructor(_name?: string, _options?: StateSwitchOptions);
    name(): string;
    version(): string;
    /**
     * @deprecated will be removed after Dec 31, 2022
     */
    setLog(logInstance?: any): void;
    /**
     * Get the current ON state (3VL).
     */
    active(): boolean | Pending;
    /**
     * Turn on the current state.
     * @param state
     *  `Pending` means we entered the turn on async process
     *  `true` means we have finished the turn on async process.
     */
    active(state: true | Pending): void;
    active(state: never): never;
    /**
     * Get the current OFF state (3VL).
     */
    inactive(): boolean | Pending;
    /**
     * Turn off the current state.
     * @param state
     *  `Pending` means we entered the turn off async process
     *  `true` means we have finished the turn off async process.
     */
    inactive(state: true | Pending): void;
    inactive(state: never): never;
    /**
     * @deprecate use `active()` instead. will be removed after Dec 31, 2022
     */
    on(state: any): any;
    /**
     * @deprecate use `inactive()` instead. will be removed after Dec 31, 2022
     */
    off(state: any): any;
    /**
     * does the state is not stable(in process)?
     */
    pending(): boolean;
    /**
     * Wait the pending state to be stable.
     */
    stable(state?: StateType, noCross?: boolean): Promise<void>;
    /**
     * @deprecated use `stable()` instead. will be removed after Dec 31, 2022
     */
    ready(): Promise<void>;
    /**
     * Huan(202105): To make RxJS fromEvent happy: type inferencing
     *  https://github.com/ReactiveX/rxjs/blob/92fbdda7c06561bc73dae3c14de3fc7aff92bbd4/src/internal/observable/fromEvent.ts#L39-L50
     */
    addEventListener(event: StateType, listener: ((payload: true | 'pending') => void)): void;
    removeEventListener(event: string, listener: ((payload: true | 'pending') => void)): void;
}
export {};
//# sourceMappingURL=state-switch.d.ts.map