/**
 *
 * StateSwitch for easy managing the states with async programming
 *
 * Class StateSwitch
 * Licenst: Apache-2.0
 * https://github.com/huan/state-switch
 *
 * Helper Class for Manage State Change
 */
import { EventEmitter } from 'events';
import { getLoggable } from 'brolog';
import { nop, isNop, } from '@pipeletteio/nop';
import { VERSION, } from './version.js';
let COUNTER = 0;
export class StateSwitch extends EventEmitter {
    _name;
    _options;
    _log;
    _activePromise;
    _inactivePromise;
    _activeResolver;
    _inactiveResolver;
    _isActive;
    _isPending;
    constructor(_name = `StateSwitch#${COUNTER++}`, _options = {}) {
        super();
        this._name = _name;
        this._options = _options;
        this._log = getLoggable(_options.log);
        this._log.verbose('StateSwitch', 'constructor(%s, "%s")', _name, JSON.stringify(_options));
        this._isActive = false;
        this._isPending = false;
        /**
         * for ready()
         */
        this._activeResolver = nop;
        this._inactiveResolver = nop;
        this._activePromise = new Promise(resolve => {
            this._activeResolver = resolve;
        });
        this._inactivePromise = Promise.resolve();
    }
    name() {
        return this._name;
    }
    version() {
        return VERSION;
    }
    /**
     * @deprecated will be removed after Dec 31, 2022
     */
    setLog(logInstance) {
        this._log = getLoggable(logInstance);
    }
    active(state) {
        /**
         * Set
         */
        if (state) {
            this._log.verbose('StateSwitch', '<%s> active(%s) <- (%s)', this._name, state, this.active());
            this._isActive = true;
            this._isPending = (state === 'pending');
            this.emit('active', state);
            /**
             * @deprecated `on` event will be removed after Dec 31, 2022
             */
            this.emit('on', state);
            /**
              * for stable()
              */
            if (isNop(this._inactiveResolver)) {
                this._inactivePromise = new Promise(resolve => (this._inactiveResolver = resolve));
            }
            if (state === true && !isNop(this._activeResolver)) {
                this._activeResolver();
                this._activeResolver = nop;
            }
            return;
        }
        /**
         * Get
         */
        const activeState = this._isActive
            ? this._isPending ? 'pending' : true
            : false;
        this._log.silly('StateSwitch', '<%s> active() is %s', this._name, activeState);
        return activeState;
    }
    inactive(state) {
        /**
         * Set
         */
        if (state) {
            this._log.verbose('StateSwitch', '<%s> inactive(%s) <- (%s)', this._name, state, this.inactive());
            this._isActive = false;
            this._isPending = (state === 'pending');
            this.emit('inactive', state);
            /**
             * @deprecated `off` event will be removed after Dec 31, 2022
             */
            this.emit('off', state);
            /**
              * for stable()
              */
            if (isNop(this._activeResolver)) {
                this._activePromise = new Promise(resolve => (this._activeResolver = resolve));
            }
            if (state === true && !isNop(this._inactiveResolver)) {
                this._inactiveResolver();
                this._inactiveResolver = nop;
            }
            return;
        }
        /**
         * Get
         */
        const inactiveState = !this._isActive
            ? this._isPending ? 'pending' : true
            : false;
        this._log.silly('StateSwitch', '<%s> inactive() is %s', this._name, inactiveState);
        return inactiveState;
    }
    /**
     * @deprecate use `active()` instead. will be removed after Dec 31, 2022
     */
    on(state) {
        this._log.error('StateSwitch', 'on() is deprecated: use active() instead.\n%s', new Error().stack);
        return this.active(state);
    }
    /**
     * @deprecate use `inactive()` instead. will be removed after Dec 31, 2022
     */
    off(state) {
        this._log.error('StateSwitch', 'off() is deprecated: use inactive() instead.\n%s', new Error().stack);
        return this.inactive(state);
    }
    /**
     * does the state is not stable(in process)?
     */
    pending() {
        this._log.silly('StateSwitch', '<%s> pending() is %s', this._name, this._isPending);
        return this._isPending;
    }
    /**
     * Wait the pending state to be stable.
     */
    async stable(state, noCross = false) {
        this._log.verbose('StateSwitch', '<%s> stable(%s, noCross=%s)', this._name, state, noCross);
        if (typeof state === 'undefined') {
            state = this._isActive ? 'active' : 'inactive';
        }
        if (state === 'active' || state === 'on') {
            if (this._isActive === false && noCross === true) {
                throw new Error('stable(active) but the state is inactive. call stable(active, false) to disable noCross');
            }
            await this._activePromise;
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        }
        else if (state === 'inactive' || state === 'off') {
            if (this._isActive === true && noCross === true) {
                throw new Error('stable(inactive) but the state is active. call stable(inactive, false) to disable noCross');
            }
            await this._inactivePromise;
        }
        else {
            throw new Error(`should not go here. ${state} should be of type 'never'`);
        }
        this._log.silly('StateSwitch', '<%s> stable(%s, %s) resolved.', this._name, state, noCross);
    }
    /**
     * @deprecated use `stable()` instead. will be removed after Dec 31, 2022
     */
    ready() {
        this._log.error('StateSwitch', 'ready() is deprecated: use stable() instead.\n%s', new Error().stack);
        return this.stable();
    }
    /**
     * Huan(202105): To make RxJS fromEvent happy: type inferencing
     *  https://github.com/ReactiveX/rxjs/blob/92fbdda7c06561bc73dae3c14de3fc7aff92bbd4/src/internal/observable/fromEvent.ts#L39-L50
     */
    addEventListener(event, listener) {
        super.addListener(event, listener);
    }
    removeEventListener(event, listener) {
        super.removeListener(event, listener);
    }
}
//# sourceMappingURL=state-switch.js.map